@@ -26,6 +26,7 @@ use teletypewriter::{create_pty_with_fork, create_pty_with_spawn};
 const DEFAULT_CONTEXT_CAPACITY: usize = 28;
 
 pub struct Context<T: EventListener> {
+    pub route_id: usize,
     pub terminal: Arc<FairMutex<Crosswords<T>>>,
     pub messenger: Messenger,
     #[cfg(not(target_os = "windows"))]
@@ -41,7 +42,6 @@ pub struct ContextManagerConfig {
     pub working_dir: Option<String>,
     pub spawn_performer: bool,
     pub use_current_path: bool,
-    pub is_collapsed: bool,
     pub is_native: bool,
     pub should_update_titles: bool,
 }
@@ -87,6 +87,8 @@ impl ContextManagerTitles {
 pub struct ContextManager<T: EventListener> {
     contexts: Vec<Context<T>>,
     current_index: usize,
+    current_route: usize,
+    acc_current_route: usize,
     #[allow(unused)]
     capacity: usize,
     event_proxy: T,
@@ -97,13 +99,19 @@ pub struct ContextManager<T: EventListener> {
 
 impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
     #[inline]
-    pub fn create_dead_context(event_proxy: T, window_id: WindowId) -> Context<T> {
+    pub fn create_dead_context(
+        event_proxy: T,
+        window_id: WindowId,
+        route_id: usize,
+    ) -> Context<T> {
         let size = CrosswordsSize::new(MIN_COLUMNS, MIN_LINES);
-        let terminal = Crosswords::new(size, CursorShape::Block, event_proxy, window_id);
+        let terminal =
+            Crosswords::new(size, CursorShape::Block, event_proxy, window_id, route_id);
         let terminal: Arc<FairMutex<Crosswords<T>>> = Arc::new(FairMutex::new(terminal));
         let (sender, _receiver) = corcovado::channel::channel();
 
         Context {
+            route_id,
             #[cfg(not(target_os = "windows"))]
             main_fd: Arc::new(-1),
             #[cfg(not(target_os = "windows"))]
@@ -118,6 +126,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         cursor_state: (&CursorState, bool),
         event_proxy: T,
         window_id: WindowId,
+        route_id: usize,
         size: SugarloafLayout,
         config: &ContextManagerConfig,
     ) -> Result<Context<T>, Box<dyn Error>> {
@@ -130,8 +139,13 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         let cols: u16 = size.columns.try_into().unwrap_or(MIN_COLUMNS as u16);
         let rows: u16 = size.lines.try_into().unwrap_or(MIN_LINES as u16);
 
-        let mut terminal =
-            Crosswords::new(size, cursor_state.0.content, event_proxy.clone(), window_id);
+        let mut terminal = Crosswords::new(
+            size,
+            cursor_state.0.content,
+            event_proxy.clone(),
+            window_id,
+            route_id,
+        );
         terminal.blinking_cursor = cursor_state.1;
         let terminal: Arc<FairMutex<Crosswords<T>>> = Arc::new(FairMutex::new(terminal));
 
@@ -185,8 +199,13 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             );
         }
 
-        let machine =
-            Machine::new(Arc::clone(&terminal), pty, event_proxy.clone(), window_id)?;
+        let machine = Machine::new(
+            Arc::clone(&terminal),
+            pty,
+            event_proxy.clone(),
+            window_id,
+            route_id,
+        )?;
         let channel = machine.channel();
         if config.spawn_performer {
             machine.spawn();
@@ -204,6 +223,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         };
 
         Ok(Context {
+            route_id,
             #[cfg(not(target_os = "windows"))]
             main_fd,
             #[cfg(not(target_os = "windows"))]
@@ -218,6 +238,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         cursor_state: (&CursorState, bool),
         event_proxy: T,
         window_id: WindowId,
+        route_id: usize,
         ctx_config: ContextManagerConfig,
         size: SugarloafLayout,
         sugarloaf_errors: Option<SugarloafErrors>,
@@ -226,6 +247,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             cursor_state,
             event_proxy.clone(),
             window_id,
+            route_id,
             size,
             &ctx_config,
         ) {
@@ -243,7 +265,11 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
                     window_id,
                 );
 
-                ContextManager::create_dead_context(event_proxy.clone(), window_id)
+                ContextManager::create_dead_context(
+                    event_proxy.clone(),
+                    window_id,
+                    route_id,
+                )
             }
         };
 
@@ -271,6 +297,8 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
 
         Ok(ContextManager {
             current_index: 0,
+            current_route: 0,
+            acc_current_route: 0,
             contexts: vec![initial_context],
             capacity: DEFAULT_CONTEXT_CAPACITY,
             event_proxy,
@@ -294,7 +322,6 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
                 args: vec![],
             },
             spawn_performer: false,
-            is_collapsed: true,
             is_native: false,
             should_update_titles: false,
             use_current_path: false,
@@ -303,6 +330,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
             (&CursorState::new('_'), false),
             event_proxy.clone(),
             window_id,
+            0,
             SugarloafLayout::default(),
             &config,
         )?;
@@ -312,6 +340,8 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
 
         Ok(ContextManager {
             current_index: 0,
+            current_route: 0,
+            acc_current_route: 0,
             contexts: vec![initial_context],
             capacity,
             event_proxy,
@@ -322,14 +352,22 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
     }
 
     #[inline]
-    #[allow(unused)]
     pub fn schedule_render(&mut self, scheduled_time: u64) {
         self.event_proxy
             .send_event(RioEvent::PrepareRender(scheduled_time), self.window_id);
     }
 
     #[inline]
-    #[allow(unused)]
+    pub fn schedule_render_on_route(&mut self, scheduled_time: u64) {
+        // PrepareRender will force a render for any route that is focused on window
+        // PrepareRenderOnRoute only call render function for specific route ids.
+        self.event_proxy.send_event(
+            RioEvent::PrepareRenderOnRoute(scheduled_time, self.current_route),
+            self.window_id,
+        );
+    }
+
+    #[inline]
     pub fn report_error_fonts_not_found(&mut self, fonts_not_found: Vec<SugarloafFont>) {
         if !fonts_not_found.is_empty() {
             self.event_proxy.send_event(
@@ -369,6 +407,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         }
 
         self.set_current(tab_index);
+        self.current_route = self.contexts[self.current_index].route_id;
     }
 
     #[inline]
@@ -410,6 +449,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         }
 
         self.set_current(self.contexts.len() - 1);
+        self.current_route = self.contexts[self.current_index].route_id;
     }
 
     #[inline]
@@ -535,6 +575,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
     pub fn close_context(&mut self) {
         if self.contexts.len() <= 1 {
             self.current_index = 0;
+            self.current_route = self.contexts[self.current_index].route_id;
             return;
         }
 
@@ -547,12 +588,14 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
 
         self.titles.titles.remove(&index_to_remove);
         self.contexts.remove(index_to_remove);
+        self.current_route = self.contexts[self.current_index].route_id;
     }
 
     #[inline]
     pub fn kill_current_context(&mut self) {
         if self.contexts.len() <= 1 {
             self.current_index = 0;
+            self.current_route = self.contexts[self.current_index].route_id;
             // In MacOS: Close last tab will work, leading to hide and
             // keep Rio running in background if allow_close_last_tab is true
             #[cfg(target_os = "macos")]
@@ -589,6 +632,11 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         self.current_index
     }
 
+    #[inline]
+    pub fn current_route(&self) -> usize {
+        self.current_route
+    }
+
     #[inline]
     pub fn current(&self) -> &Context<T> {
         &self.contexts[self.current_index]
@@ -612,6 +660,8 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         } else {
             self.current_index += 1;
         }
+
+        self.current_route = self.contexts[self.current_index].route_id;
     }
 
     #[inline]
@@ -627,6 +677,8 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
         } else {
             self.current_index -= 1;
         }
+
+        self.current_route = self.contexts[self.current_index].route_id;
     }
 
     #[inline]
@@ -675,10 +727,12 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
                 cloned_config.working_dir = working_dir;
             }
 
+            self.acc_current_route += 1;
             match ContextManager::create_context(
                 cursor_state,
                 self.event_proxy.clone(),
                 self.window_id,
+                self.acc_current_route,
                 layout,
                 &cloned_config,
             ) {
@@ -686,6 +740,7 @@ impl<T: EventListener + Clone + std::marker::Send + 'static> ContextManager<T> {
                     self.contexts.push(new_context);
                     if redirect {
                         self.current_index = last_index;
+                        self.current_route = self.contexts[self.current_index].route_id;
                     }
                 }
                 Err(..) => {
