@@ -8,6 +8,11 @@ pub fn default_line_height() -> f32 {
     1.0
 }
 
+#[inline]
+pub fn default_padding_y() -> [f32; 2] {
+    [0., 0.]
+}
+
 #[inline]
 pub fn default_shell() -> crate::config::Shell {
     #[cfg(not(target_os = "windows"))]
@@ -77,12 +82,12 @@ pub fn default_editor() -> String {
 
 #[inline]
 pub fn default_window_width() -> i32 {
-    600
+    800
 }
 
 #[inline]
 pub fn default_window_height() -> i32 {
-    400
+    500
 }
 
 #[inline]
@@ -150,6 +155,14 @@ blinking-cursor = false
 # Example:
 # padding-x = 10
 
+# Padding-y
+#
+# define y axis padding based on a format [top, left]
+# (default is [0, 0])
+#
+# Example:
+# padding-y = [30, 10]
+
 # Option as Alt
 #
 # This config only works on MacOS.
@@ -278,6 +291,12 @@ blinking-cursor = false
 # [fonts]
 # extras = [{ family = "Microsoft JhengHei" }]
 #
+# In case you want to specify any font feature:
+# [fonts]
+# features = ["ss02", "ss03", "ss05", "ss19"]
+#
+# Note: Font features do not have support to live reload on configuration,
+# so to reflect your changes, you will need to close and reopen Rio.
 #
 # Example:
 # [fonts]
