@@ -17,52 +17,65 @@ pub use crate::components::rich_text::batch::{
     Rect,
     Vertex,
 };
+use crate::components::rich_text::image_cache::glyph::GlyphCacheSession;
+use crate::components::rich_text::image_cache::ImageCache;
 pub use crate::components::rich_text::image_cache::{
     AddImage,
-    Epoch,
     ImageId,
     ImageLocation,
     TextureEvent,
     TextureId,
     // AddImage, Epoch, ImageData, ImageId, ImageLocation, TextureEvent, TextureId,
 };
-use crate::components::rich_text::image_cache::{GlyphCache, ImageCache};
 use crate::components::rich_text::text::*;
 use crate::SugarCursor;
-
 use std::borrow::Borrow;
 
+pub struct ComposedRect {
+    rect: Rect,
+    coords: [f32; 4],
+    color: [f32; 4],
+    has_alpha: bool,
+    image: TextureId,
+}
+
+pub enum CachedRect {
+    Image(ComposedRect),
+    Mask(ComposedRect),
+    Standard((Rect, [f32; 4])),
+}
+
 pub struct Compositor {
-    images: ImageCache,
-    glyphs: GlyphCache,
     batches: BatchManager,
-    epoch: Epoch,
     intercepts: Vec<(f32, f32)>,
 }
 
 impl Compositor {
     /// Creates a new compositor.
-    pub fn new(max_texture_size: u16) -> Self {
+    pub fn new() -> Self {
         Self {
-            images: ImageCache::new(max_texture_size),
-            glyphs: GlyphCache::new(),
             batches: BatchManager::new(),
-            epoch: Epoch(0),
             intercepts: Vec::new(),
         }
     }
 
     /// Advances the epoch for the compositor and clears all batches.
+    #[inline]
     pub fn begin(&mut self) {
-        self.glyphs.prune(self.epoch, &mut self.images);
-        self.epoch.0 += 1;
+        // TODO: Write a better prune system that doesn't rely on epoch
+        // self.glyphs.prune(&mut self.images);
         self.batches.reset();
     }
 
     /// Builds a display list for the current batched geometry and enumerates
     /// all texture events with the specified closure.
-    pub fn finish(&mut self, list: &mut DisplayList, events: impl FnMut(TextureEvent)) {
-        self.images.drain_events(events);
+    pub fn finish(
+        &mut self,
+        list: &mut DisplayList,
+        images: &mut ImageCache,
+        events: impl FnMut(TextureEvent),
+    ) {
+        images.drain_events(events);
         self.batches.build_display_list(list);
     }
 }
@@ -71,27 +84,35 @@ impl Compositor {
 impl Compositor {
     /// Adds an image to the compositor.
     #[allow(unused)]
-    pub fn add_image(&mut self, request: AddImage) -> Option<ImageId> {
-        self.images.allocate(self.epoch, request)
+    pub fn add_image(
+        &mut self,
+        images: &mut ImageCache,
+        request: AddImage,
+    ) -> Option<ImageId> {
+        images.allocate(request)
     }
 
     /// Returns the image associated with the specified identifier.
     #[allow(unused)]
-    pub fn get_image(&mut self, image: ImageId) -> Option<ImageLocation> {
-        self.images.get(self.epoch, image)
+    pub fn get_image(
+        &mut self,
+        images: &mut ImageCache,
+        image: ImageId,
+    ) -> Option<ImageLocation> {
+        images.get(image)
     }
 
     /// Removes the image from the compositor.
     #[allow(unused)]
-    pub fn remove_image(&mut self, image: ImageId) -> bool {
-        self.images.deallocate(image).is_some()
+    pub fn remove_image(&mut self, images: &mut ImageCache, image: ImageId) -> bool {
+        images.deallocate(image).is_some()
     }
 }
 
 /// Drawing.
 impl Compositor {
     /// Draws a rectangle with the specified depth and color.
-    #[inline]
+    #[allow(unused)]
     pub fn draw_rect(&mut self, rect: impl Into<Rect>, depth: f32, color: &[f32; 4]) {
         self.batches.add_rect(&rect.into(), depth, color);
     }
@@ -100,12 +121,13 @@ impl Compositor {
     #[allow(unused)]
     pub fn draw_image(
         &mut self,
+        images: &mut ImageCache,
         rect: impl Into<Rect>,
         depth: f32,
         color: &[f32; 4],
         image: ImageId,
     ) {
-        if let Some(img) = self.images.get(self.epoch, image) {
+        if let Some(img) = images.get(image) {
             self.batches.add_image_rect(
                 &rect.into(),
                 depth,
@@ -117,18 +139,52 @@ impl Compositor {
         }
     }
 
+    pub fn draw_glyphs_from_cache(&mut self, cache: &Vec<CachedRect>, depth: f32) {
+        for val in cache {
+            match val {
+                CachedRect::Image(data) => {
+                    self.batches.add_image_rect(
+                        &data.rect,
+                        depth,
+                        &data.color,
+                        &data.coords,
+                        data.image,
+                        data.has_alpha,
+                    );
+                }
+                CachedRect::Mask(data) => {
+                    self.batches.add_mask_rect(
+                        &data.rect,
+                        depth,
+                        &data.color,
+                        &data.coords,
+                        data.image,
+                        data.has_alpha,
+                    );
+                }
+                CachedRect::Standard((rect, bg_color)) => {
+                    self.batches.add_rect(rect, depth, bg_color);
+                }
+            }
+        }
+    }
+
     /// Draws a text run.
     pub fn draw_glyphs<I>(
         &mut self,
+        session: &mut GlyphCacheSession,
         rect: impl Into<Rect>,
         depth: f32,
         style: &TextRunStyle,
         glyphs: I,
         // dimension: SugarDimensions,
-    ) where
+    ) -> Vec<CachedRect>
+    where
         I: Iterator,
         I::Item: Borrow<Glyph>,
     {
+        // let start = std::time::Instant::now();
+
         let rect = rect.into();
         let (underline, underline_offset, underline_size, underline_color) =
             match style.underline {
@@ -143,75 +199,86 @@ impl Compositor {
         if underline {
             self.intercepts.clear();
         }
-        let mut session = self.glyphs.session(
-            self.epoch,
-            &mut self.images,
-            style.font,
-            style.font_coords,
-            style.font_size,
-        );
+        let mut result = Vec::new();
         let subpx_bias = (0.125, 0.);
         let color = style.color;
         let x = rect.x;
         for g in glyphs {
             let glyph = g.borrow();
-            let entry = session.get(glyph.id, glyph.x, glyph.y);
+            let entry = session.get(glyph.id);
             if let Some(entry) = entry {
                 if let Some(img) = session.get_image(entry.image) {
                     let gx = (glyph.x + subpx_bias.0).floor() + entry.left as f32;
                     let gy = (glyph.y + subpx_bias.1).floor() - entry.top as f32;
+
                     if entry.is_bitmap {
+                        let rect =
+                            Rect::new(gx, gy, entry.width as f32, entry.height as f32);
+                        let color = [1.0, 1.0, 1.0, 1.0];
+                        let coords = [img.min.0, img.min.1, img.max.0, img.max.1];
                         self.batches.add_image_rect(
-                            &Rect::new(gx, gy, entry.width as f32, entry.height as f32),
+                            &rect,
                             depth,
-                            &[1.0, 1.0, 1.0, 1.0],
-                            &[img.min.0, img.min.1, img.max.0, img.max.1],
+                            &color,
+                            &coords,
                             img.texture_id,
                             entry.image.has_alpha(),
                         );
+                        result.push(CachedRect::Image(ComposedRect {
+                            rect,
+                            color,
+                            coords,
+                            image: img.texture_id,
+                            has_alpha: entry.image.has_alpha(),
+                        }));
                     } else {
+                        let rect =
+                            Rect::new(gx, gy, entry.width as f32, entry.height as f32);
+                        let coords = [img.min.0, img.min.1, img.max.0, img.max.1];
                         self.batches.add_mask_rect(
-                            &Rect::new(gx, gy, entry.width as f32, entry.height as f32),
+                            &rect,
                             depth,
                             &color,
-                            &[img.min.0, img.min.1, img.max.0, img.max.1],
+                            &coords,
                             img.texture_id,
                             true,
                         );
+                        result.push(CachedRect::Mask(ComposedRect {
+                            rect,
+                            color,
+                            coords,
+                            image: img.texture_id,
+                            has_alpha: true,
+                        }));
                     }
 
                     if let Some(bg_color) = style.background_color {
-                        self.batches.add_rect(
-                            &Rect::new(
-                                rect.x,
-                                style.topline,
-                                rect.width,
-                                style.line_height,
-                            ),
-                            depth,
-                            &bg_color,
+                        let rect = Rect::new(
+                            rect.x,
+                            style.topline,
+                            rect.width,
+                            style.line_height,
                         );
+                        self.batches.add_rect(&rect, depth, &bg_color);
+                        result.push(CachedRect::Standard((rect, bg_color)));
                     }
 
                     match style.cursor {
                         SugarCursor::Block(cursor_color) => {
-                            self.batches.add_rect(
-                                &Rect::new(
-                                    rect.x,
-                                    style.topline,
-                                    rect.width,
-                                    style.line_height,
-                                ),
-                                depth,
-                                &cursor_color,
+                            let rect = Rect::new(
+                                rect.x,
+                                style.topline,
+                                rect.width,
+                                style.line_height,
                             );
+                            self.batches.add_rect(&rect, depth, &cursor_color);
+                            result.push(CachedRect::Standard((rect, cursor_color)));
                         }
                         SugarCursor::Caret(cursor_color) => {
-                            self.batches.add_rect(
-                                &Rect::new(rect.x, style.topline, 3.0, style.line_height),
-                                depth,
-                                &cursor_color,
-                            );
+                            let rect =
+                                Rect::new(rect.x, style.topline, 3.0, style.line_height);
+                            self.batches.add_rect(&rect, depth, &cursor_color);
+                            result.push(CachedRect::Standard((rect, cursor_color)));
                         }
                         _ => {}
                     }
@@ -235,22 +302,23 @@ impl Compositor {
             let uy = style.baseline - underline_offset as f32;
             for range in self.intercepts.iter() {
                 if ux < range.0 {
-                    self.batches.add_rect(
-                        &Rect::new(ux, uy, range.0 - ux, underline_size),
-                        depth,
-                        &underline_color,
-                    );
+                    let rect = Rect::new(ux, uy, range.0 - ux, underline_size);
+                    self.batches.add_rect(&rect, depth, &underline_color);
+                    result.push(CachedRect::Standard((rect, underline_color)));
                 }
                 ux = range.1;
             }
             let end = x + rect.width;
             if ux < end {
-                self.draw_rect(
-                    Rect::new(ux, uy, end - ux, underline_size),
-                    depth,
-                    &underline_color,
-                );
+                let rect = Rect::new(ux, uy, end - ux, underline_size);
+                self.batches.add_rect(&rect, depth, &underline_color);
+                result.push(CachedRect::Standard((rect, underline_color)));
             }
         }
+
+        // let duration = start.elapsed();
+        // println!(" - draw_glyphs() is: {:?}", duration);
+
+        result
     }
 }
