@@ -5,13 +5,14 @@ pub mod text;
 pub mod util;
 
 use crate::components::core::orthographic_projection;
+use crate::components::rich_text::image_cache::{GlyphCache, ImageCache};
 use crate::context::Context;
 use crate::font::FontLibraryData;
 use crate::layout::SugarDimensions;
 use compositor::{
-    Command, Compositor, DisplayList, Rect, TextureEvent, TextureId, Vertex,
+    CachedRect, Command, Compositor, DisplayList, Rect, TextureEvent, TextureId, Vertex,
 };
-use fnv::FnvHashMap;
+use rustc_hash::FxHashMap;
 use std::{borrow::Cow, mem};
 use text::{Glyph, TextRunStyle, UnderlineStyle};
 use wgpu::util::DeviceExt;
@@ -44,15 +45,18 @@ pub struct RichTextBrush {
     transform: wgpu::Buffer,
     bind_group_layout: wgpu::BindGroupLayout,
     pipeline: wgpu::RenderPipeline,
-    textures: FnvHashMap<TextureId, Texture>,
+    textures: FxHashMap<TextureId, Texture>,
     index_buffer: wgpu::Buffer,
     index_buffer_size: u64,
     current_transform: [f32; 16],
     comp: Compositor,
+    draw_layout_cache: DrawLayoutCache,
     dlist: DisplayList,
     bind_group_needs_update: bool,
     first_run: bool,
     supported_vertex_buffer: usize,
+    images: ImageCache,
+    glyphs: GlyphCache,
 }
 
 impl RichTextBrush {
@@ -216,6 +220,7 @@ impl RichTextBrush {
         });
 
         let pipeline = device.create_render_pipeline(&wgpu::RenderPipelineDescriptor {
+            cache: None,
             label: None,
             layout: Some(&pipeline_layout),
             vertex: wgpu::VertexState {
@@ -272,8 +277,11 @@ impl RichTextBrush {
             color_texture_view,
             mask_texture_view,
             sampler,
-            textures: FnvHashMap::default(),
-            comp: Compositor::new(2048),
+            textures: FxHashMap::default(),
+            comp: Compositor::new(),
+            images: ImageCache::new(2048),
+            glyphs: GlyphCache::new(),
+            draw_layout_cache: DrawLayoutCache::default(),
             dlist,
             bind_group,
             transform,
@@ -286,12 +294,19 @@ impl RichTextBrush {
         }
     }
 
+    #[inline]
+    pub fn clean_cache(&mut self) {
+        self.draw_layout_cache.clear();
+    }
+
     #[inline]
     pub fn prepare(
         &mut self,
         ctx: &mut Context,
         state: &crate::sugarloaf::state::SugarState,
     ) {
+        // let start = std::time::Instant::now();
+
         if state.compositors.advanced.render_data.is_empty() {
             self.dlist.clear();
             return;
@@ -305,15 +320,24 @@ impl RichTextBrush {
 
         draw_layout(
             &mut self.comp,
+            &mut self.images,
+            &mut self.glyphs,
             &state.compositors.advanced.render_data,
             state.current.layout.style.screen_position.0,
-            // TODO: Fix position
             state.current.layout.style.screen_position.1,
             font_library,
             state.current.layout.dimensions,
+            &mut self.draw_layout_cache,
         );
+        self.draw_layout_cache.clear_on_demand();
+        // let duration = start.elapsed();
+        // println!(" - rich_text::prepare::draw_layout() is: {:?}", duration);
+
         self.dlist.clear();
         self.finish_composition(ctx);
+
+        // let duration = start.elapsed();
+        // println!(" - rich_text::prepare() is: {:?}", duration);
     }
 
     #[inline]
@@ -328,6 +352,8 @@ impl RichTextBrush {
 
         let dimension = fetch_dimensions(
             &mut self.comp,
+            &mut self.images,
+            &mut self.glyphs,
             &state.compositors.advanced.mocked_render_data,
             font_library,
         );
@@ -345,6 +371,7 @@ impl RichTextBrush {
         state: &crate::sugarloaf::state::SugarState,
         rpass: &mut wgpu::RenderPass<'pass>,
     ) {
+        // let start = std::time::Instant::now();
         let vertices: &[Vertex] = self.dlist.vertices();
         let indices: &[u32] = self.dlist.indices();
 
@@ -513,153 +540,220 @@ impl RichTextBrush {
 
         self.bind_group_needs_update = false;
         self.first_run = false;
+
+        // let duration = start.elapsed();
+        // println!(" - rich_text::render() is: {:?}", duration);
     }
 
     #[inline]
     fn finish_composition(&mut self, ctx: &mut Context) {
-        self.comp.finish(&mut self.dlist, |event| {
-            match event {
-                TextureEvent::CreateTexture {
-                    id,
-                    format,
-                    width,
-                    height,
-                    data,
-                } => {
-                    log::info!(
-                        "rich_text::CreateTexture with id ({:?}) and format {:?}",
+        self.comp
+            .finish(&mut self.dlist, &mut self.images, |event| {
+                match event {
+                    TextureEvent::CreateTexture {
                         id,
-                        format
-                    );
-                    let texture_size = wgpu::Extent3d {
-                        width: width.into(),
-                        height: height.into(),
-                        depth_or_array_layers: 1,
-                    };
-                    let texture = ctx.device.create_texture(&wgpu::TextureDescriptor {
-                        size: texture_size,
-                        mip_level_count: 1,
-                        sample_count: 1,
-                        dimension: wgpu::TextureDimension::D2,
-                        format: match format {
-                            image_cache::PixelFormat::A8 => wgpu::TextureFormat::R8Unorm,
-                            image_cache::PixelFormat::Rgba8 => {
-                                wgpu::TextureFormat::Rgba8Unorm
-                            }
-                        },
-                        usage: wgpu::TextureUsages::TEXTURE_BINDING
-                            | wgpu::TextureUsages::COPY_DST,
-                        label: Some("rich_text::Cache"),
-                        view_formats: &[],
-                    });
-
-                    if let Some(data) = data {
-                        self.bind_group_needs_update = true;
-                        let channels = match format {
-                            // Mask
-                            image_cache::PixelFormat::A8 => 1,
-                            // Color
-                            image_cache::PixelFormat::Rgba8 => 4,
-                        };
-
-                        ctx.queue.write_texture(
-                            // Tells wgpu where to copy the pixel data
-                            wgpu::ImageCopyTexture {
-                                texture: &texture,
-                                mip_level: 0,
-                                origin: wgpu::Origin3d::ZERO,
-                                aspect: wgpu::TextureAspect::All,
-                            },
-                            // The actual pixel data
-                            data,
-                            // The layout of the texture
-                            wgpu::ImageDataLayout {
-                                offset: 0,
-                                bytes_per_row: Some((width * channels).into()),
-                                rows_per_image: Some(height.into()),
-                            },
-                            texture_size,
+                        format,
+                        width,
+                        height,
+                        data,
+                    } => {
+                        log::info!(
+                            "rich_text::CreateTexture with id ({:?}) and format {:?}",
+                            id,
+                            format
                         );
-                    }
-
-                    self.textures.insert(id, texture);
-                }
-                TextureEvent::UpdateTexture {
-                    id,
-                    format,
-                    x,
-                    y,
-                    width,
-                    height,
-                    data,
-                } => {
-                    log::info!("rich_text::UpdateTexture id ({:?})", id);
-                    if let Some(texture) = self.textures.get(&id) {
-                        self.bind_group_needs_update = true;
                         let texture_size = wgpu::Extent3d {
                             width: width.into(),
                             height: height.into(),
                             depth_or_array_layers: 1,
                         };
-
-                        let channels = match format {
-                            // Mask
-                            image_cache::PixelFormat::A8 => 1,
-                            // Color
-                            image_cache::PixelFormat::Rgba8 => 4,
-                        };
-
-                        ctx.queue.write_texture(
-                            // Tells wgpu where to copy the pixel data
-                            wgpu::ImageCopyTexture {
-                                texture,
-                                mip_level: 0,
-                                origin: wgpu::Origin3d {
-                                    x: u32::from(x),
-                                    y: u32::from(y),
-                                    z: 0,
+                        let texture =
+                            ctx.device.create_texture(&wgpu::TextureDescriptor {
+                                size: texture_size,
+                                mip_level_count: 1,
+                                sample_count: 1,
+                                dimension: wgpu::TextureDimension::D2,
+                                format: match format {
+                                    image_cache::PixelFormat::A8 => {
+                                        wgpu::TextureFormat::R8Unorm
+                                    }
+                                    image_cache::PixelFormat::Rgba8 => {
+                                        wgpu::TextureFormat::Rgba8Unorm
+                                    }
                                 },
-                                aspect: wgpu::TextureAspect::All,
-                            },
-                            // The actual pixel data
-                            data,
-                            // The layout of the texture
-                            wgpu::ImageDataLayout {
-                                offset: 0,
-                                bytes_per_row: Some((width * channels).into()),
-                                rows_per_image: Some(height.into()),
-                            },
-                            texture_size,
-                        );
+                                usage: wgpu::TextureUsages::TEXTURE_BINDING
+                                    | wgpu::TextureUsages::COPY_DST,
+                                label: Some("rich_text::Cache"),
+                                view_formats: &[],
+                            });
+
+                        if let Some(data) = data {
+                            self.bind_group_needs_update = true;
+                            let channels = match format {
+                                // Mask
+                                image_cache::PixelFormat::A8 => 1,
+                                // Color
+                                image_cache::PixelFormat::Rgba8 => 4,
+                            };
+
+                            ctx.queue.write_texture(
+                                // Tells wgpu where to copy the pixel data
+                                wgpu::ImageCopyTexture {
+                                    texture: &texture,
+                                    mip_level: 0,
+                                    origin: wgpu::Origin3d::ZERO,
+                                    aspect: wgpu::TextureAspect::All,
+                                },
+                                // The actual pixel data
+                                data,
+                                // The layout of the texture
+                                wgpu::ImageDataLayout {
+                                    offset: 0,
+                                    bytes_per_row: Some((width * channels).into()),
+                                    rows_per_image: Some(height.into()),
+                                },
+                                texture_size,
+                            );
+                        }
+
+                        self.textures.insert(id, texture);
+                    }
+                    TextureEvent::UpdateTexture {
+                        id,
+                        format,
+                        x,
+                        y,
+                        width,
+                        height,
+                        data,
+                    } => {
+                        log::info!("rich_text::UpdateTexture id ({:?})", id);
+                        if let Some(texture) = self.textures.get(&id) {
+                            self.bind_group_needs_update = true;
+                            let texture_size = wgpu::Extent3d {
+                                width: width.into(),
+                                height: height.into(),
+                                depth_or_array_layers: 1,
+                            };
+
+                            let channels = match format {
+                                // Mask
+                                image_cache::PixelFormat::A8 => 1,
+                                // Color
+                                image_cache::PixelFormat::Rgba8 => 4,
+                            };
+
+                            ctx.queue.write_texture(
+                                // Tells wgpu where to copy the pixel data
+                                wgpu::ImageCopyTexture {
+                                    texture,
+                                    mip_level: 0,
+                                    origin: wgpu::Origin3d {
+                                        x: u32::from(x),
+                                        y: u32::from(y),
+                                        z: 0,
+                                    },
+                                    aspect: wgpu::TextureAspect::All,
+                                },
+                                // The actual pixel data
+                                data,
+                                // The layout of the texture
+                                wgpu::ImageDataLayout {
+                                    offset: 0,
+                                    bytes_per_row: Some((width * channels).into()),
+                                    rows_per_image: Some(height.into()),
+                                },
+                                texture_size,
+                            );
+                        }
+                    }
+                    TextureEvent::DestroyTexture(id) => {
+                        log::info!("rich_text::DestroyTexture id ({:?})", id);
+                        self.textures.remove(&id);
                     }
                 }
-                TextureEvent::DestroyTexture(id) => {
-                    log::info!("rich_text::DestroyTexture id ({:?})", id);
-                    self.textures.remove(&id);
-                }
-            }
-        });
+            });
+    }
+}
+
+#[derive(Default)]
+struct DrawLayoutCache {
+    inner: FxHashMap<String, Vec<CachedRect>>,
+}
+
+impl DrawLayoutCache {
+    #[inline]
+    fn get(&self, id: &str) -> Option<&Vec<CachedRect>> {
+        self.inner.get(id)
+    }
+
+    #[inline]
+    fn insert(&mut self, id: String, data: Vec<CachedRect>) {
+        self.inner.insert(id, data);
+    }
+
+    #[inline]
+    fn clear_on_demand(&mut self) {
+        if self.inner.len() > 1024 {
+            self.inner.clear();
+        }
+    }
+
+    #[inline]
+    fn clear(&mut self) {
+        self.inner.clear();
     }
 }
 
 #[inline]
+#[allow(clippy::too_many_arguments)]
 fn draw_layout(
     comp: &mut compositor::Compositor,
+    images: &mut ImageCache,
+    glyphs_cache: &mut GlyphCache,
     render_data: &crate::layout::RenderData,
     x: f32,
     y: f32,
     font_library: &FontLibraryData,
-    _rect: SugarDimensions,
+    rect: SugarDimensions,
+    draw_layout_cache: &mut DrawLayoutCache,
 ) {
+    // let start = std::time::Instant::now();
     let depth = 0.0;
     let mut glyphs = Vec::new();
+    let mut current_font = 0;
+    let mut current_font_size = 0.0;
+    let mut current_font_coords: Vec<i16> = Vec::with_capacity(0);
+    if let Some(line) = render_data.lines().next() {
+        if let Some(run) = line.runs().next() {
+            current_font = *run.font();
+            current_font_size = run.font_size();
+            current_font_coords = run.normalized_coords().to_vec();
+        }
+    }
+
+    let mut session = glyphs_cache.session(
+        images,
+        font_library[current_font].as_ref(),
+        &current_font_coords,
+        current_font_size,
+    );
+
     for line in render_data.lines() {
+        let hash = line.hash().unwrap_or(0);
         let mut px = x + line.offset();
-        for run in line.runs() {
-            let mut font = *run.font();
-            if font == 0 {
-                font = run.font_id_based_on_attr();
+        let id = format!("{}-{}-{}", hash, line.baseline(), px);
+        if hash > 0 {
+            if let Some(data) = draw_layout_cache.get(&id) {
+                comp.draw_glyphs_from_cache(data, depth);
+                continue;
             }
+        }
+
+        let mut cache = Vec::default();
+        for run in line.runs() {
+            let font = *run.font();
+            let char_width = run.char_width();
 
             let py = line.baseline() + y;
             let run_x = px;
@@ -668,19 +762,19 @@ fn draw_layout(
                 for glyph in cluster.glyphs() {
                     let x = px + glyph.x;
                     let y = py - glyph.y;
-                    px += glyph.advance;
-                    // px += rect.width;
+                    // px += glyph.advance;
+                    px += rect.width * char_width;
+                    // cache.glyphs.push(Glyph { id: glyph.id, x, y });
                     glyphs.push(Glyph { id: glyph.id, x, y });
                 }
             }
-            let color = run.color();
 
             let line_height = line.ascent() + line.descent() + line.leading();
             let style = TextRunStyle {
                 font: font_library[font].as_ref(),
                 font_coords: run.normalized_coords(),
                 font_size: run.font_size(),
-                color,
+                color: run.color(),
                 cursor: run.cursor(),
                 background_color: run.background_color(),
                 baseline: py,
@@ -698,24 +792,71 @@ fn draw_layout(
                 },
             };
 
-            comp.draw_glyphs(
-                Rect::new(run_x, py, style.advance, 1.),
-                depth,
-                &style,
-                glyphs.iter(),
-            );
+            if font != current_font
+                || style.font_size != current_font_size
+                || style.font_coords != current_font_coords
+            {
+                session = glyphs_cache.session(
+                    images,
+                    style.font,
+                    style.font_coords,
+                    style.font_size,
+                );
+
+                current_font = font;
+                current_font_coords = style.font_coords.to_vec();
+                current_font_size = style.font_size;
+            }
+
+            if hash > 0 {
+                cache.extend(comp.draw_glyphs(
+                    &mut session,
+                    Rect::new(run_x, py, style.advance, 1.),
+                    depth,
+                    &style,
+                    glyphs.iter(),
+                ));
+            }
+        }
+
+        if hash > 0 && !cache.is_empty() {
+            draw_layout_cache.insert(id, cache);
         }
     }
+
+    // let duration = start.elapsed();
+    // println!(" - draw_layout() is: {:?}\n", duration);
 }
 
 #[inline]
 fn fetch_dimensions(
     comp: &mut compositor::Compositor,
+    images: &mut ImageCache,
+    glyphs_cache: &mut GlyphCache,
     render_data: &crate::layout::RenderData,
     font_library: &FontLibraryData,
 ) -> SugarDimensions {
     let x = 0.;
     let y = 0.;
+
+    let mut current_font = 0;
+    let mut current_font_size = 0.0;
+    let mut current_font_coords: Vec<i16> = Vec::with_capacity(0);
+    if let Some(line) = render_data.lines().next() {
+        if let Some(run) = line.runs().next() {
+            current_font = *run.font();
+            current_font_size = run.font_size();
+            current_font_coords = run.normalized_coords().to_vec();
+        }
+    }
+
+    let mut session = glyphs_cache.session(
+        images,
+        font_library[current_font].as_ref(),
+        &current_font_coords,
+        current_font_size,
+    );
+
     let mut glyphs = Vec::with_capacity(3);
     let mut dimension = SugarDimensions::default();
     for line in render_data.lines() {
@@ -755,7 +896,24 @@ fn fetch_dimensions(
                 dimension.height = line_height;
             }
 
+            if font != &current_font
+                || style.font_size != current_font_size
+                || style.font_coords != current_font_coords
+            {
+                session = glyphs_cache.session(
+                    images,
+                    style.font,
+                    style.font_coords,
+                    style.font_size,
+                );
+
+                current_font = *font;
+                current_font_coords = style.font_coords.to_vec();
+                current_font_size = style.font_size;
+            }
+
             comp.draw_glyphs(
+                &mut session,
                 Rect::new(run_x, py, style.advance, 1.),
                 0.0,
                 &style,
