@@ -7,32 +7,23 @@
 // nav and span_style were originally retired from dfrg/swash_demo licensed under MIT
 // https://github.com/dfrg/swash_demo/blob/master/LICENSE
 
-//! Experimental paragraph layout engine.
-
-mod bidi;
 mod builder;
 mod builder_data;
 mod content;
 mod layout_data;
-mod line_breaker;
 mod render_data;
-mod span_style;
 
 pub use content::{Content, ContentBuilder};
 pub use render_data::RenderData;
 
-#[doc(inline)]
-pub use swash::text::Language;
-
 /// Iterators over elements of a paragraph.
 pub mod iter {
     pub use super::render_data::{Clusters, Glyphs, Lines, Runs};
 }
 
 pub use builder::{LayoutContext, ParagraphBuilder};
-pub use line_breaker::{Alignment, BreakLines};
+pub use builder_data::FragmentStyle;
 pub use render_data::{Cluster, Glyph, Line, Run};
-pub use span_style::*;
 
 /// Largest allowable span or fragment identifier.
 const MAX_ID: usize = i32::MAX as usize;
@@ -123,7 +114,7 @@ fn compute(
     margin: Delta<f32>,
 ) -> (usize, usize) {
     let margin_x = ((margin.x) * dimensions.scale).floor();
-    let margin_spaces = (margin.top_y * 2.) + margin.bottom_y;
+    let margin_spaces = margin.top_y + margin.bottom_y;
 
     let mut lines = (height / dimensions.scale) - margin_spaces;
     lines /= (dimensions.height / dimensions.scale) * line_height;
@@ -253,11 +244,6 @@ impl SugarloafLayout {
         }
     }
 
-    #[inline]
-    pub fn set_margin_top_y(&mut self, top_y: f32) {
-        self.margin.top_y = top_y;
-    }
-
     // This method will run over the new font and font_size
     #[inline]
     pub fn recalculate(
@@ -298,6 +284,7 @@ impl SugarloafLayout {
         if should_apply_changes {
             update_styles(self);
         }
+
         self
     }
 }
