@@ -11,11 +11,6 @@
 
 //! RenderData.
 use super::layout_data::*;
-use super::line_breaker::BreakLines;
-use super::Direction;
-use crate::font::{
-    Style, Weight, FONT_ID_BOLD, FONT_ID_BOLD_ITALIC, FONT_ID_ITALIC, FONT_ID_REGULAR,
-};
 use crate::layout::FragmentStyle;
 use crate::sugarloaf::primitives::SugarCursor;
 use core::iter::DoubleEndedIterator;
@@ -95,11 +90,9 @@ pub struct CachedRunData {
     pub coords: Vec<i16>,
     pub span: FragmentStyle,
     pub line: u32,
+    pub hash: Option<u64>,
     pub font: usize,
     pub size: f32,
-    pub level: u8,
-    pub whitespace: bool,
-    pub trailing_whitespace: bool,
     pub ascent: f32,
     pub descent: f32,
     pub leading: f32,
@@ -162,11 +155,9 @@ impl RenderData {
                 clusters: (clusters_start, clusters_end),
                 span: cached_run.span,
                 line,
+                hash: cached_run.hash,
                 font: cached_run.font,
                 size: cached_run.size,
-                level: cached_run.level,
-                whitespace: false,
-                trailing_whitespace: false,
                 ascent: cached_run.ascent,
                 descent: cached_run.descent,
                 leading: cached_run.leading,
@@ -179,13 +170,14 @@ impl RenderData {
         self.data.last_span = 0;
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub(super) fn push_run(
         &mut self,
         styles: &[FragmentStyle],
         font: &usize,
         size: f32,
-        level: u8,
         line: u32,
+        hash: Option<u64>,
         shaper: Shaper<'_>,
     ) {
         // In case is a new line,
@@ -224,12 +216,10 @@ impl RenderData {
                     let run_data = RunData {
                         span: styles[last_span],
                         line,
+                        hash,
                         font: *font,
                         coords: (coords_start, coords_end),
                         size,
-                        level,
-                        whitespace: false,
-                        trailing_whitespace: false,
                         clusters: (clusters_start, clusters_end),
                         ascent: metrics.ascent * span_data.line_spacing,
                         descent: metrics.descent * span_data.line_spacing,
@@ -270,12 +260,10 @@ impl RenderData {
                     self.last_cached_run.runs.push(CachedRunData {
                         span: styles[last_span],
                         line,
+                        hash,
                         font: *font,
                         coords: coords.to_owned(),
                         size,
-                        level,
-                        whitespace: false,
-                        trailing_whitespace: false,
                         clusters: owned_clusters,
                         ascent: metrics.ascent * span_data.line_spacing,
                         descent: metrics.descent * span_data.line_spacing,
@@ -361,12 +349,10 @@ impl RenderData {
         let run_data = RunData {
             span: styles[last_span],
             line,
+            hash,
             font: *font,
             coords: (coords_start, coords_end),
             size,
-            level,
-            whitespace: false,
-            trailing_whitespace: false,
             clusters: (clusters_start, clusters_end),
             ascent: metrics.ascent * span_data.line_spacing,
             descent: metrics.descent * span_data.line_spacing,
@@ -405,12 +391,10 @@ impl RenderData {
         self.last_cached_run.runs.push(CachedRunData {
             span: styles[last_span],
             line,
+            hash,
             font: *font,
             coords: coords.to_owned(),
             size,
-            level,
-            whitespace: false,
-            trailing_whitespace: false,
             clusters: owned_clusters,
             ascent: metrics.ascent * span_data.line_spacing,
             descent: metrics.descent * span_data.line_spacing,
@@ -443,50 +427,6 @@ impl RenderData {
             size: glyph.data as usize,
         });
     }
-
-    pub(super) fn apply_spacing(&mut self) {
-        for run in &mut self.data.runs {
-            let word = run.span.word_spacing;
-            let letter = run.span.letter_spacing;
-            if word == 0. && letter == 0. {
-                continue;
-            }
-            let clusters =
-                &mut self.data.clusters[run.clusters.0 as usize..run.clusters.1 as usize];
-            for cluster in clusters {
-                let mut spacing = letter;
-                if word != 0. && cluster.info.whitespace().is_space_or_nbsp() {
-                    spacing += word;
-                }
-                if spacing != 0. {
-                    let detailed_glyphs = &mut self.data.detailed_glyphs[..];
-                    if cluster.is_detailed() && !cluster.is_ligature() {
-                        self.data.detailed_clusters[cluster.glyphs as usize].advance +=
-                            spacing;
-                    } else if cluster.is_last_continuation() {
-                        cluster.glyphs =
-                            (f32::from_bits(cluster.glyphs) + spacing).to_bits();
-                    }
-                    if let Some(g) = cluster
-                        .glyphs_mut(&self.data.detailed_clusters, &mut self.data.glyphs)
-                        .last_mut()
-                    {
-                        if g.is_simple() {
-                            g.add_spacing(spacing);
-                        } else {
-                            detailed_glyphs[g.detail_index()].advance += spacing;
-                        }
-                        run.advance += spacing;
-                    }
-                }
-            }
-        }
-    }
-
-    pub(super) fn finish(&mut self) {
-        // Zero out the advance for the extra trailing space.
-        self.data.glyphs.last_mut().unwrap().clear_advance();
-    }
 }
 
 /// Sequence of clusters sharing the same font, size and span.
@@ -497,31 +437,12 @@ pub struct Run<'a> {
 }
 
 impl<'a> Run<'a> {
-    pub(super) fn new(layout: &'a LayoutData, run: &'a RunData) -> Self {
-        Self { layout, run }
-    }
     /// Returns the span that contains the run.
     #[inline]
     pub fn span(&self) -> FragmentStyle {
         self.run.span
     }
 
-    #[inline]
-    pub fn font_id_based_on_attr(&self) -> usize {
-        let is_italic = self.run.span.font_attrs.2 == Style::Italic;
-        let is_bold = self.run.span.font_attrs.1 == Weight::BOLD;
-
-        if is_bold && is_italic {
-            return FONT_ID_BOLD_ITALIC;
-        } else if is_bold {
-            return FONT_ID_BOLD;
-        } else if is_italic {
-            return FONT_ID_ITALIC;
-        }
-
-        FONT_ID_REGULAR
-    }
-
     /// Returns the font for the run.
     #[inline]
     pub fn font(&self) -> &usize {
@@ -540,10 +461,9 @@ impl<'a> Run<'a> {
         self.run.span.color
     }
 
-    /// Returns the bidi level of the run.
     #[inline]
-    pub fn level(&self) -> u8 {
-        self.run.level
+    pub fn char_width(&self) -> f32 {
+        self.run.span.width
     }
 
     /// Returns the cursor
@@ -552,15 +472,6 @@ impl<'a> Run<'a> {
         self.run.span.cursor
     }
 
-    /// Returns the direction of the run.
-    pub fn direction(&self) -> Direction {
-        if self.run.level & 1 != 0 {
-            Direction::RightToLeft
-        } else {
-            Direction::LeftToRight
-        }
-    }
-
     /// Returns the normalized variation coordinates for the run.
     pub fn normalized_coords(&self) -> &'a [NormalizedCoord] {
         self.layout
@@ -622,11 +533,10 @@ impl<'a> Run<'a> {
     /// Returns an iterator over the clusters in visual order.
     #[inline]
     pub fn visual_clusters(&self) -> Clusters<'a> {
-        let rev = self.run.level & 1 != 0;
         Clusters {
             layout: self.layout,
             iter: self.layout.clusters[make_range(self.run.clusters)].iter(),
-            rev,
+            rev: false,
         }
     }
 }
@@ -735,13 +645,7 @@ pub struct Cluster<'a> {
     cluster: ClusterData,
 }
 
-// gar.content.width().unwrap_or(
-
 impl<'a> Cluster<'a> {
-    // pub(super) fn new(layout: &'a LayoutData, cluster: ClusterData) -> Self {
-    //     Self { layout, cluster }
-    // }
-
     /// Returns the cluster information.
     #[inline]
     pub fn info(&self) -> ClusterInfo {
@@ -864,14 +768,6 @@ pub struct Line<'a> {
 }
 
 impl<'a> Line<'a> {
-    // pub(super) fn new(layout: &'a RenderData, line_index: usize) -> Self {
-    //     Self {
-    //         layout: &layout.data,
-    //         line_layout: &layout.line_data,
-    //         line: &layout.line_data.lines[line_index],
-    //     }
-    // }
-
     /// Returns the offset in line direction.
     #[inline]
     pub fn offset(&self) -> f32 {
@@ -908,30 +804,6 @@ impl<'a> Line<'a> {
         self.line.width
     }
 
-    /// Returns the total advance of the line excluding trailing whitespace.
-    // pub fn advance_without_trailing_whitespace(&self) -> f32 {
-    //     let mut advance = self.line.width;
-    //     for run in self.line_layout.runs[make_range(self.line.runs)]
-    //         .iter()
-    //         .rev()
-    //     {
-    //         if !run.trailing_whitespace {
-    //             break;
-    //         }
-    //         for cluster in self.layout.clusters[make_range(run.clusters)].iter().rev() {
-    //             if !cluster.info.is_whitespace() {
-    //                 break;
-    //             }
-    //             advance -= Cluster {
-    //                 layout: self.layout,
-    //                 cluster: *cluster,
-    //             }
-    //             .advance();
-    //         }
-    //     }
-    //     advance
-    // }
-
     /// Returns the size of the line (height for horizontal and width
     /// for vertical layouts).
     #[inline]
@@ -949,9 +821,10 @@ impl<'a> Line<'a> {
         }
     }
 
-    // pub(super) fn data(&self) -> &'a LineData {
-    //     self.line
-    // }
+    #[inline]
+    pub fn hash(&self) -> &Option<u64> {
+        &self.line.hash
+    }
 }
 
 /// Iterator over the lines of a paragraph.
