@@ -67,6 +67,7 @@ pub struct SugarloafWindow {
 pub struct SugarloafRenderer {
     pub power_preference: wgpu::PowerPreference,
     pub backend: wgpu::Backends,
+    pub font_features: Option<Vec<String>>,
 }
 
 impl Default for SugarloafRenderer {
@@ -79,6 +80,7 @@ impl Default for SugarloafRenderer {
         SugarloafRenderer {
             power_preference: wgpu::PowerPreference::HighPerformance,
             backend: default_backend,
+            font_features: None,
         }
     }
 }
@@ -117,6 +119,7 @@ impl Sugarloaf<'_> {
         font_library: &FontLibrary,
         layout: SugarloafLayout,
     ) -> Result<Sugarloaf<'a>, SugarloafWithErrors<'a>> {
+        let font_features = renderer.font_features.to_owned();
         let ctx = Context::new(window, renderer).await;
 
         let text_brush = {
@@ -128,8 +131,7 @@ impl Sugarloaf<'_> {
         let rect_brush = RectBrush::init(&ctx);
         let layer_brush = LayerBrush::new(&ctx);
         let rich_text_brush = RichTextBrush::new(&ctx);
-
-        let state = SugarState::new(layout, font_library);
+        let state = SugarState::new(layout, font_library, &font_features);
 
         let instance = Sugarloaf {
             state,
@@ -150,7 +152,6 @@ impl Sugarloaf<'_> {
         log::info!("requested a font change");
 
         self.state.reset_compositor();
-        self.state.compositors.advanced.clean();
         self.state.set_fonts(font_library);
     }
 
@@ -174,6 +175,11 @@ impl Sugarloaf<'_> {
         self.state.next.layout
     }
 
+    #[inline]
+    pub fn layout_next_mut(&mut self) -> &mut SugarloafLayout {
+        &mut self.state.next.layout
+    }
+
     #[inline]
     pub fn update_font_size(&mut self, operation: u8) {
         self.state.compute_layout_font_size(operation);
@@ -215,16 +221,6 @@ impl Sugarloaf<'_> {
         self.state.insert_on_current_line(sugar);
     }
 
-    #[inline]
-    pub fn insert_on_current_line_from_vec(&mut self, sugar_vec: &Vec<&Sugar>) {
-        self.state.insert_on_current_line_from_vec(sugar_vec);
-    }
-
-    #[inline]
-    pub fn insert_on_current_line_from_vec_owned(&mut self, sugar_vec: &Vec<Sugar>) {
-        self.state.insert_on_current_line_from_vec_owned(sugar_vec);
-    }
-
     #[inline]
     pub fn finish_line(&mut self) {
         self.state.compute_line_end();
@@ -284,11 +280,6 @@ impl Sugarloaf<'_> {
         self.state.graphics.remove(graphic_id);
     }
 
-    #[inline]
-    pub fn dimensions_changed(&self) -> bool {
-        self.state.dimensions_changed()
-    }
-
     #[inline]
     pub fn mark_dirty(&mut self) {
         self.state.is_dirty = true;
